<?php

/**
 * ODataController.php
 *
 * A sample Controller for the extension odataclient
 *
 * @link http://www.odata.org/
 * @link http://odataphp.codeplex.com/
 *
 * @author Joe Blocher <yii@myticket.at>
 * @copyright 2011 myticket it-solutions gmbh
 * @license New BSD License
 * @category Webservice
 * @package odataclient
 * @version 0.1
 */

Yii::import('ext.odataclient.components.*');

class ODataController extends Controller
{

	/**
	 * Override the default viewPath
	 *
	 * @return string
	 */
	public function getViewPath()
	{
		return dirname(__FILE__) . '/views/odata';
	}

	/**
	 * Generate the service proxy class file for the specified service
	 *
	 * @param string $svc the seriveNameAlias
	 * @param string $config the config file name
	 */
	public function actionGenProxyFile($svc,$config=null)
	{
		//don't autogenerate the service proxy class file
		//autogenerate is in silent mode and will not override existing files
		$client = new EODataClient($svc,$config,false);
		$client->generateProxyClassFile();
	}

	/**
	 * Retrieve data from the specified service/$entityset
	 * or list list the available entitySets
	 *
	 * @param string $svc
	 * @param mixed $entityset
	 * @return
	 */
	public function actionServices($svc,$entityset=null)
	{
		$client = new EODataClient($svc);

		if (isset($entityset)) //query the entitySet
		{
			$entitySet = $client->createEntitySet($entityset);

			//query param = null means, get all data of the entitySet
			$dataProvider = new EODataClientDataprovider($client,$entityset,null,
								array(
										  'pagination'=>array(
										  	'pageSize'=>5,
										  ),
									));

			$this->render('services',array(
			                              'dataProvider'=>$dataProvider,
			                            ));

		}
		else //list the available entitySets from the service $svc
		{
			$dataProvider = $client->getEntitiesDataProvider(array(
												         'pagination'=>array(
															   'pageSize'=>5,
															)));
			$this->render('entitysets',array('dataProvider'=>$dataProvider,'client'=>$client));
		}
	}

	/**
	 * List the configured odata service from config.php
	 */
	public function actionIndex()
	{
	  $dataProvider = EODataClient::getConfiguredServicesDataProvider();
	  $this->render('index',array('dataProvider'=>$dataProvider));
	}

	/**
	 * Show the OData PHP SDK User Guide
	 */
	public function actionUserGuide()
	{
		$userGuide = Yii::getPathOfAlias('ext.odataclient.vendors.OData_PHP_SDK.doc.User_Guide').'.htm';
		include($userGuide);
		Yii::app()->end();
	}

	/**
	 * Execute the code from SimpleApplications Sample1.php (slightly modified)
	 * See the sample code how to work with the SDK
	 */
	public function actionSample1()
	{
		//import the SDK and the service proxy class NorthwindEntities
		EODataClient::importServices('ext.odataclient.services.classes.examples.NorthwindEntities');

		$sampleCode1 = Yii::getPathOfAlias('ext.odataclient.vendors.OData_PHP_SDK.samples.PHPApplications.SimpleApplications.Sample1').'.php';
        include($sampleCode1);
	}

	/**
	 * This is the sample code from SimpleApplications Sample1.php
	 * It uses the EODataClient to create the query. This is the only difference
	 */
	public function actionSample2()
	{
		$client = new EODataClient('northwind');

		echo "<h3>Sample1: List all cutomer's ID in NorthWind DB with USA as Country using Server Side Paging and print the number of coustmers using inline count</h3>";
		try
		{
			$query = $client->createEntitySet('Customers')->filter("Country eq 'USA'")
			                                     ->IncludeTotalCount();


			$customersResponse = $query->Execute();

			echo "<br/>";
			echo "Number of Customers with Country as USA:" . $customersResponse->TotalCount() . "<br/>";
			echo "CustomerID" . "<br/>";
			echo "----------" . "<br/>";
			$nextCustomerToken = null;

			do
			{
				if($nextCustomerToken != null)
				{
					$customersResponse = $svc->Execute($nextCustomerToken);
				}

				foreach($customersResponse->Result as $customer)
				{
					echo $customer->CustomerID . "<br/>";
				}

			}while(($nextCustomerToken = $customersResponse->GetContinuation()) != null);

		}
		catch(DataServiceRequestException $ex)
		{
			echo 'Error: while running the query ' . $ex->Response->getQuery();
			echo "<br/>";
			echo $ex->Response->getError();
		}
		catch (ODataServiceException $e)
		{
			echo "Error:" . $e->getError() . "<br>" . "Detailed Error:" . $e->getDetailedError();
		}
	}
}